@@ -1,4 +1,7 @@
 import iso8601
+import pytz
+from tzlocal import get_localzone
+from datetime import datetime
 
 
 def parse_date(input_value):
@@ -23,3 +26,11 @@ def parse_float(input_value, default_value=None):
         return float(input_value)
     except ValueError:
         return default_value
+
+
+def localize_datetime(timestamp: datetime) -> datetime:
+    if timestamp.tzinfo is not None:
+        return timestamp
+
+    local_tz = get_localzone()
+    return local_tz.localize(timestamp).astimezone(pytz.utc)
