@@ -8,19 +8,20 @@ import responses
 from requests.exceptions import HTTPError
 
 from pygrocy import Grocy
-from pygrocy.grocy import Chore, Product, Group, ShoppingListProduct
-from pygrocy.grocy_api_client import GrocyApiClient, UserDto, \
-    ProductData
+from pygrocy.grocy import Chore, Product, Group, ShoppingListProduct, AssignmentType
+from pygrocy.grocy_api_client import GrocyApiClient, UserDto, ProductData
 from test.test_const import CONST_BASE_URL, CONST_PORT, CONST_SSL
 
 
 class TestGrocy(TestCase):
-
     def setUp(self):
         self.grocy_regular = Grocy(CONST_BASE_URL, "api_key")
-        self.grocy = Grocy(CONST_BASE_URL, "demo_mode", verify_ssl=CONST_SSL, port=CONST_PORT)
+        self.grocy = Grocy(
+            CONST_BASE_URL, "demo_mode", verify_ssl=CONST_SSL, port=CONST_PORT
+        )
         self.base_url = f"{CONST_BASE_URL}:{CONST_PORT}/api"
-        self.date_test = datetime.strptime("2019-05-04 11:31:04", '%Y-%m-%d %H:%M:%S')
+        self.date_test = datetime.strptime("2019-05-04 11:31:04", "%Y-%m-%d %H:%M:%S")
+        self.add_generic_data = {"name": "This is a task"}
 
     def test_init(self):
         self.assertIsInstance(self.grocy, Grocy)
@@ -31,7 +32,7 @@ class TestGrocy(TestCase):
 
         assert len(tasks) == 6
         assert tasks[0].id == 1
-        assert tasks[0].name == 'Repair the garage door'
+        assert tasks[0].name == "Repair the garage door"
 
     @unittest.skip("no chores_current table in current demo data")
     def test_get_chores_valid(self):
@@ -60,7 +61,7 @@ class TestGrocy(TestCase):
                 "period_config": null,
                 "track_date_only": "0",
                 "rollover": "0",
-                "assignment_type": null,
+                "assignment_type": "who-least-did-first",
                 "assignment_config": null,
                 "next_execution_assigned_to_user_id": null,
                 "consume_product_on_execution": "0",
@@ -73,9 +74,14 @@ class TestGrocy(TestCase):
             "next_execution_assigned_user": null
         }"""
         details_json = json.loads(details_json)
-        responses.add(responses.GET, f"{self.base_url}/chores/1", json=details_json, status=200)
+        responses.add(
+            responses.GET, f"{self.base_url}/chores/1", json=details_json, status=200
+        )
         chore_details = self.grocy.chore(1)
         self.assertIsInstance(chore_details, Chore)
+        self.assertEqual(
+            chore_details.assignment_type, AssignmentType.WHO_LEAST_DID_FIRST
+        )
 
     @unittest.skip("no stock_current table in current demo data")
     def test_product_get_details_valid(self):
@@ -83,7 +89,9 @@ class TestGrocy(TestCase):
 
         product = stock[0]
 
-        api_client = GrocyApiClient(CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL)
+        api_client = GrocyApiClient(
+            CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL
+        )
         product.get_details(api_client)
 
         self.assertIsInstance(product.name, str)
@@ -134,13 +142,20 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_shopping_list_invalid_no_data(self):
-        responses.add(responses.GET, f"{self.base_url}/objects/shopping_list", status=400)
+        responses.add(
+            responses.GET, f"{self.base_url}/objects/shopping_list", status=400
+        )
         self.assertRaises(HTTPError, self.grocy.shopping_list)
 
     @responses.activate
     def test_get_shopping_list_invalid_missing_data(self):
         resp = []
-        responses.add(responses.GET, f"{self.base_url}/objects/shopping_list", json=resp, status=200)
+        responses.add(
+            responses.GET,
+            f"{self.base_url}/objects/shopping_list",
+            json=resp,
+            status=200,
+        )
         self.assertEqual(len(self.grocy.shopping_list()), 0)
 
     @unittest.skip("no shopping list existing in current demo data")
@@ -149,7 +164,11 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_add_missing_product_to_shopping_list_error(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/shoppinglist/add-missing-products", status=400)
+        responses.add(
+            responses.POST,
+            f"{self.base_url}/stock/shoppinglist/add-missing-products",
+            status=400,
+        )
         self.assertRaises(HTTPError, self.grocy.add_missing_product_to_shopping_list)
 
     @unittest.skip("no shopping list existing in current demo data")
@@ -162,22 +181,34 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_clear_shopping_list_valid(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/shoppinglist/clear", status=204)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/shoppinglist/clear", status=204
+        )
         self.grocy.clear_shopping_list()
 
     @responses.activate
     def test_clear_shopping_list_error(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/shoppinglist/clear", status=400)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/shoppinglist/clear", status=400
+        )
         self.assertRaises(HTTPError, self.grocy.clear_shopping_list)
 
     @responses.activate
     def test_remove_product_in_shopping_list_valid(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/shoppinglist/remove-product", status=204)
+        responses.add(
+            responses.POST,
+            f"{self.base_url}/stock/shoppinglist/remove-product",
+            status=204,
+        )
         self.grocy.remove_product_in_shopping_list(1)
 
     @responses.activate
     def test_remove_product_in_shopping_list_error(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/shoppinglist/remove-product", status=400)
+        responses.add(
+            responses.POST,
+            f"{self.base_url}/stock/shoppinglist/remove-product",
+            status=400,
+        )
         self.assertRaises(HTTPError, self.grocy.remove_product_in_shopping_list, 1)
 
     @unittest.skip("no userentities table in current demo data")
@@ -195,22 +226,35 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_product_groups_invalid_no_data(self):
-        responses.add(responses.GET, f"{self.base_url}/objects/product_groups", status=400)
+        responses.add(
+            responses.GET, f"{self.base_url}/objects/product_groups", status=400
+        )
         self.assertRaises(HTTPError, self.grocy.product_groups)
 
     @responses.activate
     def test_get_product_groups_invalid_missing_data(self):
         resp = []
-        responses.add(responses.GET, f"{self.base_url}/objects/product_groups", json=resp, status=200)
+        responses.add(
+            responses.GET,
+            f"{self.base_url}/objects/product_groups",
+            json=resp,
+            status=200,
+        )
         self.assertEqual(len(self.grocy.product_groups()), 0)
 
     @responses.activate
     def test_add_product_pic_valid(self):
         with patch("os.path.exists") as m_exist:
-            with patch("builtins.open", mock_open()) as m_open:
+            with patch("builtins.open", mock_open()):
                 m_exist.return_value = True
-                responses.add(responses.PUT, f"{self.base_url}/files/productpictures/MS5qcGc=", status=204)
-                responses.add(responses.PUT, f"{self.base_url}/objects/products/1", status=204)
+                responses.add(
+                    responses.PUT,
+                    f"{self.base_url}/files/productpictures/MS5qcGc=",
+                    status=204,
+                )
+                responses.add(
+                    responses.PUT, f"{self.base_url}/objects/products/1", status=204
+                )
                 resp = self.grocy.add_product_pic(1, "/somepath/pic.jpg")
                 self.assertIsNone(resp)
 
@@ -218,20 +262,32 @@ class TestGrocy(TestCase):
     def test_add_product_pic_invalid_missing_data(self):
         with patch("os.path.exists") as m_exist:
             m_exist.return_value = False
-            self.assertRaises(FileNotFoundError, self.grocy.add_product_pic, 1, "/somepath/pic.jpg")
+            self.assertRaises(
+                FileNotFoundError, self.grocy.add_product_pic, 1, "/somepath/pic.jpg"
+            )
 
     @responses.activate
     def test_upload_product_picture_error(self):
         with patch("os.path.exists") as m_exist:
-            with patch("builtins.open", mock_open()) as m_open:
+            with patch("builtins.open", mock_open()):
                 m_exist.return_value = True
-                api_client = GrocyApiClient(CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL)
-                responses.add(responses.PUT, f"{self.base_url}/files/productpictures/MS5qcGc=", status=400)
-                self.assertRaises(HTTPError, api_client.upload_product_picture, 1, "/somepath/pic.jpg")
+                api_client = GrocyApiClient(
+                    CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL
+                )
+                responses.add(
+                    responses.PUT,
+                    f"{self.base_url}/files/productpictures/MS5qcGc=",
+                    status=400,
+                )
+                self.assertRaises(
+                    HTTPError, api_client.upload_product_picture, 1, "/somepath/pic.jpg"
+                )
 
     @responses.activate
     def test_update_product_pic_error(self):
-        api_client = GrocyApiClient(CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL)
+        api_client = GrocyApiClient(
+            CONST_BASE_URL, "demo_mode", port=CONST_PORT, verify_ssl=CONST_SSL
+        )
         responses.add(responses.PUT, f"{self.base_url}/objects/products/1", status=400)
         self.assertRaises(HTTPError, api_client.update_product_pic, 1)
 
@@ -247,19 +303,19 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_expiring_invalid_no_data(self):
-        resp = {
-            "expiring_products": [],
-            "expired_products": [],
-            "missing_products": []
-        }
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        resp = {"expiring_products": [], "expired_products": [], "missing_products": []}
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
         self.grocy.expiring_products(True)
 
     @responses.activate
     def test_get_expiring_invalid_missing_data(self):
         resp = {}
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
     @unittest.skip("no stock_current table in current demo data")
     def test_get_expired_products_valid(self):
@@ -273,19 +329,19 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_expired_invalid_no_data(self):
-        resp = {
-            "expiring_products": [],
-            "expired_products": [],
-            "missing_products": []
-        }
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        resp = {"expiring_products": [], "expired_products": [], "missing_products": []}
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
         self.grocy.expired_products(True)
 
     @responses.activate
     def test_get_expired_invalid_missing_data(self):
         resp = {}
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
     @unittest.skip("no stock_current table in current demo data")
     def test_get_missing_products_valid(self):
@@ -301,32 +357,31 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_missing_invalid_no_data(self):
-        resp = {
-            "expiring_products": [],
-            "expired_products": [],
-            "missing_products": []
-        }
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        resp = {"expiring_products": [], "expired_products": [], "missing_products": []}
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
         self.grocy.missing_products(True)
 
     @responses.activate
     def test_get_missing_invalid_missing_data(self):
         resp = {}
-        responses.add(responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200)
+        responses.add(
+            responses.GET, f"{self.base_url}/stock/volatile", json=resp, status=200
+        )
 
     @responses.activate
     def test_get_userfields_valid(self):
-        resp = {
-            "uf1": 0,
-            "uf2": "string"
-        }
+        resp = {"uf1": 0, "uf2": "string"}
 
-        responses.add(responses.GET, f"{self.base_url}/userfields/chores/1", json=resp, status=200)
+        responses.add(
+            responses.GET, f"{self.base_url}/userfields/chores/1", json=resp, status=200
+        )
 
         a_chore_uf = self.grocy.get_userfields("chores", 1)
 
-        self.assertEqual(a_chore_uf['uf1'], 0)
+        self.assertEqual(a_chore_uf["uf1"], 0)
 
     @unittest.skip("no userentities table in current demo data")
     def test_get_userfields_invalid_no_data(self):
@@ -340,7 +395,9 @@ class TestGrocy(TestCase):
     @responses.activate
     def test_set_userfields_error(self):
         responses.add(responses.PUT, f"{self.base_url}/userfields/chores/1", status=400)
-        self.assertRaises(HTTPError, self.grocy.set_userfields, "chores", 1, "auserfield", "value")
+        self.assertRaises(
+            HTTPError, self.grocy.set_userfields, "chores", 1, "auserfield", "value"
+        )
 
     def test_get_last_db_changed_valid(self):
 
@@ -351,28 +408,59 @@ class TestGrocy(TestCase):
     @responses.activate
     def test_get_last_db_changed_invalid_no_data(self):
         resp = {}
-        responses.add(responses.GET, f"{self.base_url}/system/db-changed-time", json=resp, status=200)
+        responses.add(
+            responses.GET,
+            f"{self.base_url}/system/db-changed-time",
+            json=resp,
+            status=200,
+        )
 
         self.assertIsNone(self.grocy.get_last_db_changed())
 
     @responses.activate
     def test_add_product_valid(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/products/1/add", status=200)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/products/1/add", status=200
+        )
         self.assertIsNone(self.grocy.add_product(1, 1.3, 2.44, self.date_test))
 
     @responses.activate
     def test_add_product_error(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/products/1/add", status=400)
-        self.assertRaises(HTTPError, self.grocy.add_product, 1, 1.3, 2.44, self.date_test)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/products/1/add", status=400
+        )
+        self.assertRaises(
+            HTTPError, self.grocy.add_product, 1, 1.3, 2.44, self.date_test
+        )
+
+    @responses.activate
+    def test_add_generic_valid(self):
+        responses.add(
+            responses.POST, f"{self.base_url}/objects/tasks", status=200
+        )
+        self.assertIsNone(self.grocy.add_generic("tasks", self.add_generic_data))
+
+    @responses.activate
+    def test_add_generic_error(self):
+        responses.add(
+            responses.POST, f"{self.base_url}/objects/tasks", status=400
+        )
+        self.assertRaises(
+            HTTPError, self.grocy.add_generic, "tasks", self.add_generic_data
+        )
 
     @responses.activate
     def test_consume_product_valid(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/products/1/consume", status=200)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/products/1/consume", status=200
+        )
         self.assertIsNone(self.grocy.consume_product(1, 1.3, self.date_test))
 
     @responses.activate
     def test_consume_product_error(self):
-        responses.add(responses.POST, f"{self.base_url}/stock/products/1/consume", status=400)
+        responses.add(
+            responses.POST, f"{self.base_url}/stock/products/1/consume", status=400
+        )
         self.assertRaises(HTTPError, self.grocy.consume_product, 1, 1.3, self.date_test)
 
     @responses.activate
@@ -387,7 +475,8 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_meal_plan(self):
-        resp_json = json.loads("""[
+        resp_json = json.loads(
+            """[
               {
                 "id": "1",
                 "day": "2020-08-10",
@@ -401,8 +490,14 @@ class TestGrocy(TestCase):
                 "row_created_timestamp": "2020-08-12 19:59:30",
                 "userfields": null
               }
-          ]""")
-        responses.add(responses.GET, f"{self.base_url}/objects/meal_plan", json=resp_json, status=200)
+          ]"""
+        )
+        responses.add(
+            responses.GET,
+            f"{self.base_url}/objects/meal_plan",
+            json=resp_json,
+            status=200,
+        )
         meal_plan = self.grocy.meal_plan()
         self.assertEqual(len(meal_plan), 1)
         self.assertEqual(meal_plan[0].id, 1)
@@ -410,7 +505,8 @@ class TestGrocy(TestCase):
 
     @responses.activate
     def test_get_recipe(self):
-        resp_json = json.loads("""{
+        resp_json = json.loads(
+            """{
           "id": "1",
           "name": "Pizza",
           "description": "<p>Mix everything</p>",
@@ -422,8 +518,14 @@ class TestGrocy(TestCase):
           "type": "normal",
           "product_id": "",
           "userfields": null
-        }""")
-        responses.add(responses.GET, f"{self.base_url}/objects/recipes/1", json=resp_json, status=200)
+        }"""
+        )
+        responses.add(
+            responses.GET,
+            f"{self.base_url}/objects/recipes/1",
+            json=resp_json,
+            status=200,
+        )
         recipe = self.grocy.recipe(1)
         self.assertEqual(recipe.id, 1)
         self.assertEqual(recipe.name, "Pizza")
